function [ Fs ] = kernelest( Y, X, AXES, WAXES, bw, THRESH )
	YX = [ Y X ];
	
	[ naxes kaxes ] = size( AXES );

	f4     = zeros( naxes, 1 );
	f2     = zeros( naxes, 1 );
	df4dy1 = zeros( naxes, 1 );
	df4dy2 = zeros( naxes, 1 );
	df4dx1 = zeros( naxes, 1 );
	df4dx2 = zeros( naxes, 1 );
	df2dx1 = zeros( naxes, 1 );
	df2dx2 = zeros( naxes, 1 );

	for ( i = 1 : naxes )
		axisbb = AXES(i,:); 
		
		f2(i)     = joint( X, axisbb(3:4), bw.fX );        
		f4(i)     = joint( YX, axisbb, bw.fYX );        
		df4dy1(i) = djoint( YX, axisbb, bw.dfYXdy1, 1 );    
		df4dy2(i) = djoint( YX, axisbb, bw.dfYXdy2, 2 );    
		df4dx1(i) = djoint( YX, axisbb, bw.dfYXdx1, 3 );    
		df4dx2(i) = djoint( YX, axisbb, bw.dfYXdx2, 4 );    
		df2dx1(i) = djoint( X, axisbb(3:4), bw.dfXdx1, 1 ); 
		df2dx2(i) = djoint( X, axisbb(3:4), bw.dfXdx2, 2 ); 
	end

	fcond  = f4 ./ f2;
	dfdy1  = df4dy1 ./ f2;
	dfdy2  = df4dy2 ./ f2;
	dfdx1  = df4dx1 ./ f2 - df2dx1 .* f4 ./ f2 .^ 2;
	dfdx2  = df4dx2 ./ f2 - df2dx2 .* f4 ./ f2 .^ 2;

	
	ZeroWeight = ( f4 > THRESH & f2 > THRESH & fcond > THRESH ) .* WAXES;
  
    Fs      = struct(                   ...
		'fYX',      f4 ,        ...
		'fY_X',     fcond ,     ...
		'fX',       f2 ,        ...
		'dfYXdx1',  df4dx1 ,    ...
		'dfY_Xdx1', dfdx1 ,     ...
		'dfYXdx2',  df4dx2 ,    ...
		'dfY_Xdx2', dfdx2 ,     ...
		'dfYXdy1',  df4dy1 ,    ...
		'dfY_Xdy1', dfdy1 ,     ...
		'dfYXdy2',  df4dy2 ,    ...
		'dfY_Xdy2', dfdy2 ,     ...
		'dfXdx1',   df2dx1 ,    ...
		'dfXdx2',   df2dx2 ,    ...
		'weight',   ZeroWeight  ...
	);
end



