%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
close all;
clear all;

DGP_LINEAR = 1;         
DGP_NONLIN = 2;         
dgp        = DGP_LINEAR; % set dgp through this variable

 
Gx     = 25;     % number of (x) linear grid points (full grid is Gx ^ 2)
N      = 500;    % number of observations per simulation
order  = 8;      % kernel order; 
rseed  = 999;    % random seed 
S      = 500;    % number of simulations
THRESH = 1e-10;  % what is the minimum threshold for estimated densities to be included

 
N4d0 = N ^ ( -1 / ( 4 + 2 * order ) );
N4d1 = N ^ ( -1 / ( 4 + 2 * order + 2 ) );
N2d0 = N ^ ( -1 / ( 2 + 2 * order ) );
N2d1 = N ^ ( -1 / ( 2 + 2 * order + 2 ) );
 

if ( dgp == DGP_LINEAR )
	 
    dgpcode  = 'LINEAR';
    alpha    =  0.75;
    beta     = -0.5;
    Astar    = [ 1 -alpha; -beta 1 ];
    invAstar = eye(2)/Astar;
    
    truer1y1 = 1;
    truer1y2 = -alpha;
    truer2y1 = -beta;
    truer2y2 = 1;
      
elseif ( dgp == DGP_NONLIN )
	dgpcode  = 'NONLIN';
     
    Gamma    = 2;
    gammainv = Gamma^(-1);
    C        = 10;
    d        = 5;
    options  = optimset( 'TolFun', 1e-20, 'TolX', 1e-20, 'display', 'off' );
    y0       = [ 5 5 ];  
    delta1   = 4.5;
    delta2   = .1;
 
    truer1y1 = -.2;
    truer1y2 = -1.;
    truer2y1 = -0.1;
    truer2y2 = 1.;

end

mue1     = 0;
mue2     = 0;
sigmae11 = 1;
sigmae22 = 1;
sigmae12 = 0;
vare     = [ sigmae11 sigmae12; sigmae12 sigmae22 ];

mup1     = 0;
mup2     = 0;
sigmap11 = 1;
sigmap22 = 1;
sigmap12 = 0;
varp     = [ sigmap11 sigmap12; sigmap12 sigmap22 ];
 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DATA GENERATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

necess = S * N;
 
filename = sprintf( 'data_%s.mat', dgpcode );
 
if  ( exist( filename ))
	load( filename );
    fprintf( 'Loading data ... \n' );
    if ( size( PP, 1 ) < necess )
	fprintf( 'Data in file not sufficient... \n' );
    break;
    end
else
    randn( 'state', rseed );
	YY = zeros([necess,1]);
    fprintf( 'Making new  data ... \n' );
	
    eps = mvnrnd( repmat([ mue1 mue2 ],[necess,1]), vare );
    PP  = mvnrnd( repmat([ mup1 mup2 ],[necess,1]), varp );
	
	if ( dgp == DGP_LINEAR )
		YY = ( eps - PP ) * invAstar'; 
	elseif ( dgp == DGP_NONLIN )
        for ( i = 1:necess )
            YY(i,1) = fzero( @(y1) (delta2 * y1) - (gammainv * log((C/y1)-1)) + (eps(i,1)-PP(i,1)+eps(i,2)-PP(i,2)+delta1-d), [.00000000001  9.99999999999],options);    
        end
        YY(:,2) = delta1 * ones([necess,1]) + delta2 * YY(:,1) ...
                                                     - PP(:,2) + eps(:,2);
    else
		error( 'Bad DGP specified; must be LINEAR or NONLIN.' );
	end
	save( filename, 'PP', 'YY' );
end


p1S = reshape( PP(1:necess,1), [ N necess / N ] )';
p2S = reshape( PP(1:necess,2), [ N necess / N ] )';
y1S = reshape( YY(1:necess,1), [ N necess / N ] )';
y2S = reshape( YY(1:necess,2), [ N necess / N ] )';

fprintf( 'Data generation complete!\n' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ESTIMATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ( dgp == DGP_LINEAR )
	y1axis = 0;
	y2axis = 0;
elseif ( dgp == DGP_NONLIN )
	y1axis = 5;
	y2axis = 5;
end

 
    x1axis = linspace( mup1 -  sqrt( sigmap11 ), mup1 +  sqrt( sigmap11 ), Gx );
	x2axis = linspace( mup2 -  sqrt( sigmap22 ), mup2 +  sqrt( sigmap22 ), Gx );

Gx1 = length( x1axis );
Gx2 = length( x2axis );
G   = struct( 'x1', Gx1, 'x2', Gx2);


AXES = [kron(y1axis, ones([Gx1*Gx2,1]))  kron(y2axis,ones([Gx1*Gx2,1])) ...
        kron(x1axis',ones([Gx2,1]))      repmat(x2axis',[Gx1,1])];

WAXES = normpdf(AXES(:,3),0,sqrt(sigmap11)/3).* normpdf(AXES(:,4),0,sqrt(sigmap22)/3);
 

estr1y1      = zeros( [S,1] );
estr1y2      = zeros( [S,1] );
estr2y1      = zeros( [S,1] );
estr2y2      = zeros( [S,1] );
 
estvarr1y1   = zeros( [S,1] );
estvarr1y2   = zeros( [S,1] );
estvarr2y1   = zeros( [S,1] );
estvarr2y2   = zeros( [S,1] );
 
eststdr1y1   = zeros( [S,1] );
eststdr1y2   = zeros( [S,1] );
eststdr2y1   = zeros( [S,1] );
eststdr2y2   = zeros( [S,1] );

bd1          = zeros( [S,1] );
bd2          = zeros( [S,1] );
 

for ( s = 1 : S )

	p = [ p1S(s,:)' p2S(s,:)' ];
	y = [ y1S(s,:)' y2S(s,:)' ];

    std4 = [std(y) std(p)];
    std2 = std(p);

bands = struct(            ...
	'fYX',     std4 * N4d0, ...
	'fX',      std2 * N2d0, ...
	'dfYXdy1', std4 * N4d1, ...
	'dfYXdy2', std4 * N4d1, ...
	'dfYXdx1', std4 * N4d1, ...
	'dfYXdx2', std4 * N4d1, ...
	'dfXdx1',  std2 * N2d1, ...
	'dfXdx2',  std2 * N2d1 ...
);

	Fs = kernelest( y, p, AXES, WAXES, bands, THRESH );

	[ estryS estvarS ] = npryvar( Fs, x1axis, x2axis );
    
    estr1y1(s) = estryS(1,1);
    estr1y2(s) = estryS(1,2);
    estr2y1(s) = estryS(2,1);
    estr2y2(s) = estryS(2,2);
    
    estvarr1y1(s) = estvarS(1,1);
    estvarr2y1(s) = estvarS(2,2);
    estvarr1y2(s) = estvarS(3,3);
    estvarr2y2(s) = estvarS(4,4);
    
    
    bd1(s) = (bands.dfYXdy1(1)^3) * (bands.dfYXdy1(2)  );
    bd2(s) = (bands.dfYXdy2(1)  ) * (bands.dfYXdy2(2)^3);
               
    eststdr1y1(s) = sqrt( estvarr1y1(s) / (N * bd1(s)) );
    eststdr1y2(s) = sqrt( estvarr1y2(s) / (N * bd2(s)) );
    eststdr2y1(s) = sqrt( estvarr2y1(s) / (N * bd1(s)) );
    eststdr2y2(s) = sqrt( estvarr2y2(s) / (N * bd2(s)) );

end
    
    meanestr1y1 = mean(estr1y1(1:S));
    meanestr1y2 = mean(estr1y2(1:S));
    meanestr2y1 = mean(estr2y1(1:S));
    meanestr2y2 = mean(estr2y2(1:S));

    stdestr1y1 = std(estr1y1(1:S));
    stdestr1y2 = std(estr1y2(1:S));
    stdestr2y1 = std(estr2y1(1:S));
    stdestr2y2 = std(estr2y2(1:S));

    meaneststdr1y1 = mean(eststdr1y1(1:S));
    meaneststdr1y2 = mean(eststdr1y2(1:S));
    meaneststdr2y1 = mean(eststdr2y1(1:S));
    meaneststdr2y2 = mean(eststdr2y2(1:S));
    
    cover95r1y1 = sum(  (truer1y1 > (estr1y1(1:S) - 1.96 * eststdr1y1(1:S))) ...
                      & (truer1y1 < (estr1y1(1:S) + 1.96 * eststdr1y1(1:S))) ) /S; 
    cover95r1y2 = sum(  (truer1y2 > (estr1y2(1:S) - 1.96 * eststdr1y2(1:S))) ...
                      & (truer1y2 < (estr1y2(1:S) + 1.96 * eststdr1y2(1:S))) ) /S;
    cover95r2y1 = sum(  (truer2y1 > (estr2y1(1:S) - 1.96 * eststdr2y1(1:S))) ...
                      & (truer2y1 < (estr2y1(1:S) + 1.96 * eststdr2y1(1:S))) ) /S; 
    cover95r2y2 = sum(  (truer2y2 > (estr2y2(1:S) - 1.96 * eststdr2y2(1:S))) ...
                      & (truer2y2 < (estr2y2(1:S) + 1.96 * eststdr2y2(1:S))) ) /S;
    
    

