clc;
close all;
clear all;
 
DGP_LINEAR = 1;         
DGP_NONLIN = 2;         
dgp        = DGP_NONLIN;

Gx     = 1000;  % number of (x) linear grid points (full grid is Gx ^ 2)
N      = 5000;  % number of observations per simulation
order  = 8;     % kernel order; 
rseed  = 999;   % random seed 
S      = 500;   % number of simulations
THRESH = 1e-3;

N3d0 = N ^ ( -1 / ( 3 + 2 * order ) );
N3d1 = N ^ ( -1 / ( 3 + 2 * order + 2 ) );
N1d0 = N ^ ( -1 / ( 1 + 2 * order ) );
N1d1 = N ^ ( -1 / ( 1 + 2 * order + 2 ) );
 
if ( dgp == DGP_LINEAR )
	dgpcode   = 'LINEAR';
    truedm    = .75;
    y00       = [0 0];
elseif ( dgp == DGP_NONLIN )
	dgpcode   = 'NONLIN';
    truedm    = -5.;
    y00       = [5 5];
else
	error( 'Bad DGP specified; must be LINEAR or NONLIN.' );
end

% DGP PARAMETERS

alpha    =  0.75;
beta     = -0.5;
Astar    = [ 1 -alpha; -beta 1 ];
invAstar = eye(2)/Astar;

Gamma    = 2;
C        = 10;
d        = 5;
options  = optimset( 'TolFun', 1e-20, 'TolX', 1e-20, 'display', 'off' );
delta1   = 4.5;
delta2   = .1;

    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DATA GENERATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

necess = S * N;
 
filename = sprintf( 'data_%s.mat', dgpcode );
 
if  ( exist( filename ))
	load( filename );
    fprintf( 'Loading data ... \n' );
    if ( size( PP, 1 ) < necess )
	fprintf( 'Data in file not sufficient... \n' );
    break;
    end
else
    randn( 'state', rseed );
	YY = zeros([necess,2]);
    fprintf( 'Making new  data ... \n' );
	
    eps1  = normrnd(0,.5,[necess,1]);
 
    ma = -1.* (1+ .1 * eps1).^2;
    mb =  1.* (1+ .2 * eps1).^2;
    va =  .1 * (1+ .5 * eps1).^2;
    vb =  .1 * (1+  2 * eps1).^2;
    
    eps2a = normrnd(ma,va);
    eps2b = normrnd(mb,vb);
    
    pe2   = [eps2a eps2b ];
    ze2   = unidrnd(2,[necess,1]);
    we2   = [ (ze2==1) (ze2==2) ];
    pe2   =  pe2 .* we2;
 
    eps2  =  sum(pe2,2);
     
    eps   = [eps1 eps2];
     
    PP    = unifrnd(-1.5,1.5,[necess,1]);

	if ( dgp == DGP_LINEAR )
		YY = ( eps - [zeros([necess,1]) PP] ) * invAstar'; 
	elseif ( dgp == DGP_NONLIN )
        gammainv = Gamma^(-1);
        for ( i = 1:necess )
            YY(i,1) = fzero( @(y1) (delta2 * y1) - (gammainv * log((C/y1)-1)) + (eps(i,1) +eps(i,2)-PP(i)+delta1-d), [.00000000001  9.99999999999],options);    
        end
        YY(:,2) = delta1 * ones([necess,1]) + delta2 * YY(:,1) ...
                                                     - PP + eps(:,2);
    else
		error( 'Bad DGP specified; must be LINEAR or NONLIN.' );
	end
	save( filename, 'PP', 'YY' );
end


p2S = reshape( PP(1:necess), [ N S ] )';
y1S = reshape( YY(1:necess,1), [ N S ] )';
y2S = reshape( YY(1:necess,2), [ N S ] )';

fprintf( 'Data generation complete!\n' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ESTIMATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


   
estdm     = zeros( [S,1] );
estvar    = zeros( [S,1] );
eststd    = zeros( [S,1] );
bd        = zeros( [S,1] );
constest  = zeros( [S,1] );


for ( s = 1 : S )

	p = [ p2S(s,:)' ];
	y = [ y1S(s,:)' y2S(s,:)' ];
    
    % standarize the data to be able to use same bandwidths for all coordinates
    
    meanp = mean(p);
    meany = mean(y);
    
    sigp = std(p);
    sigy = std(y);

    %  set axes 
    
    y0     = (y00 - meany)./std(y);
    y1axis = y0(1);
    y2axis = y0(2);
      
    p = (p - repmat(meanp,[N,1]))./repmat(sigp,[N,1]);
    y = (y - repmat(meany,[N,1]))./repmat(sigy,[N,1]);
    
    x2axis = linspace( quantile(p,.05), quantile(p,.95), Gx );
 
    AXES = [y1axis * ones([Gx,1]) y2axis * ones([Gx,1]) x2axis'];

    WAXES = (1/(quantile(p,.95)-quantile(p,.05))) * ((AXES(:,3)< quantile(p,.95)) & (AXES(:,3) > quantile(p,.05)) ) ;

 
	% keep constest for adjusting estimate and variance to original data
    
    constest(s) = sigy(1) / sigy(2);
    % specify bandwidths
    
    std3 = [std(y) std(p)];
    std1 = std(p);

    bands = struct(            ...
        'fYX',     std3 * N3d0, ...
        'fX',      std1 * N1d0, ...
        'dfYXdy1', std3 * N3d1, ...
        'dfYXdy2', std3 * N3d1, ...
        'dfYXdx1', std3 * N3d1, ...
        'dfYXdx2', std3 * N3d1, ...
        'dfXdx1',  std1 * N1d1, ...
        'dfXdx2',  std1 * N1d1 ...
    );


	Fs = kernelest_oneinst( y, p, AXES, WAXES, bands, THRESH );

	[ estdmS estvardmS estdiff ] = npryvar_oneinst( Fs, x2axis );
   
    estdm(s)  = estdmS * constest(s);
    estvar(s) = estvardmS * (constest(s)^2);
     
    bd(s) = (bands.dfYXdy1(1)^2) * (bands.dfYXdy1(2)^2 );
               
    eststd(s) = sqrt( estvar(s) / (N * bd(s)) );
    

end
    
    meanest    = mean(estdm);
    
    stdest     = std(estdm);

    meaneststd = mean(eststd);

    cover95    = sum( (truedm > (estdm - 1.96 * eststd)) ...
                      & (truedm < (estdm + 1.96 * eststd)) ) /S;
                   
