function [ dlogs ] = estimate_dlogs( YX, AXES, order, bw)


    nyx   = size( YX   ,1 );
	naxes = size( AXES, 1 );
    
    f3     = zeros( naxes, 1 );
	f1     = zeros( naxes, 1 );

    df3dy1 = zeros( naxes, 1 );
	df3dy2 = zeros( naxes, 1 );
    df3dx = zeros( naxes, 1 );
    df1dx = zeros( naxes, 1 );

    df3dy1dx = zeros( naxes, 1 );
    df3dy2dx = zeros( naxes, 1 );
    
    df3dxdx = zeros( naxes, 1 );
     
    df1dxdx = zeros( naxes, 1 );

    % For each point in the axes obtain estimated densities and derivativves
    
for ( i = 1 : naxes )
 
     axisbb = AXES(i,:);   
        
     dat1i  = YX(:,3)   - repmat(axisbb(3),[nyx 1]);
     dat3i  = YX        - repmat(axisbb   ,[nyx 1]);
        
    f1(i)     = djoint( dat1i, bw.fX ,  0      , order );         % estimate fX
  	f3(i)     = djoint( dat3i, bw.fYX, [0 0 0] , order );         % estimate fYX
	
        
    df3dy1(i) = djoint( dat3i, bw.dfYXdy1, [1 0 0 ], order );  
	df3dy2(i) = djoint( dat3i, bw.dfYXdy2, [0 1 0 ], order );  
    df3dx(i)  = djoint( dat3i, bw.dfYXdx,  [0 0 1 ], order );  
 
    df1dx(i)  = djoint( dat1i, bw.dfXdx ,  1, order );

    df3dy1dx(i) = djoint( dat3i, bw.dfYXdy1dx, [1 0 1], order );  
    df3dy2dx(i) = djoint( dat3i, bw.dfYXdy2dx, [0 1 1], order );  
    df3dxdx(i)  = djoint( dat3i, bw.dfYXdxdx,  [0 0 2], order );  
     
    df1dxdx(i) = djoint( dat1i, bw.dfXdxdx,    2, order );
    
end
     
    
    dlogfconddy1  = df3dy1 ./ f3;
    dlogfconddy2  = df3dy2 ./ f3;
    dlogfconddx   = df3dx  ./ f3 - df1dx ./ f1;
     
    dlogfconddy1dx = df3dy1dx ./ f3 - df3dy1 .* df3dx ./ f3 .^ 2;
    dlogfconddy2dx = df3dy2dx ./ f3 - df3dy2 .* df3dx ./ f3 .^ 2;
    dlogfconddxdx  = df3dxdx  ./ f3 - df3dx  .* df3dx ./ f3 .^ 2  ...
                   - df1dxdx  ./ f1 + df1dx  .* df1dx ./ f1 .^ 2;
    
    
    
    dlogs = struct(  ...
            'f3'   , f3,                    ...
            'dy1'  , dlogfconddy1,  ...
            'dy2'  , dlogfconddy2,  ...
            'dx'   , dlogfconddx,  ...
            'dy1dx', dlogfconddy1dx ,  ...
            'dy2dx', dlogfconddy2dx ,  ...
            'dxdx' , dlogfconddxdx     ...
                     );

end

