clc;
close all;
clear all;
 
DGP_LINEAR = 1;         
DGP_NONLIN = 2;         
dgp        = DGP_NONLIN; % set dgp through this variable

loadeps = 0;   % loadeps = 1 if file eps exists, loadeps = 0 otherwise

% Simulation parameters

N      = 25000;   % number of observations per simulation
S      = 500;     % number of simulations
rseed  = 999;     % random seed 

if ( dgp == DGP_LINEAR )
	dgpcode   = 'LINEAR';
elseif ( dgp == DGP_NONLIN )
	dgpcode   = 'NONLIN';
else
	error( 'Bad DGP specified; must be LINEAR or NONLIN.' );
end


% Linear DGP parameters

alpha    =  0.75;
beta     = -0.5;
Astar    = [ 1 -alpha; -beta 1 ];
invAstar = eye(2)/Astar;

% Nonlinear DGP parameters
 
Gamma    = 2;
C        = 10;
d        = 5;
options  = optimset( 'TolFun', 1e-20, 'TolX', 1e-20, 'display', 'off' );
y0       = [ 5 5 ]; 
delta1   = 4.5;
delta2   = (y0(2)-delta1)/y0(1);
 

% DATA GENERATION

necess = S * N;
 
filename = sprintf( 'data_%s.mat', dgpcode );
 
    if ~(loadeps)
    randn( 'state', rseed );
    lP = -1; 
    uP =  1;
    le2 = -1;
    ue2 = 1;
    le1 = -1;
    ue1 = 1;
    
	YY = zeros([necess,2]);
     
    fprintf( 'Making new  data ... \n' );
	
    eps1 = unifrnd(-1,1,[necess,1]);
     
    qeps2 = unifrnd(0,1,[necess,1]);
    eps2 = zeros([necess, 1]);
    
    % generate eps2 by first getting an approx to cdf of fe2conde1
    
    grid     = ( le2 : 0.001 : ue2 );
    sizegrid = length(grid);
                    
    logfe2cond = @( e1,e2 ) ((.25 * e2.^2) - (.5 * e2.^4) + e1 + e1 * e2 + e1 * e2.^2 );
 
    for ( ie = 1 : necess )
         e1         = eps1(ie);

            % calculate constant
            
            z  = linspace(le2, ue2, sizegrid);
            fe2conde1 = (exp(logfe2cond(e1,z))-min(exp(logfe2cond(e1,z)))) ;
     
            constfe2 = ( sizegrid / ((ue2-le2)*sum(fe2conde1)) ) ;
            
            % find e2
             
            fe2conde1  = fe2conde1 * constfe2;
  
            cdfe2conde1 = fe2conde1 * triu( ones( length( grid ) ) ) * ((ue2-le2)/sizegrid) ;
            
            onesqeps2   = qeps2(ie) * ones([1,sizegrid]);
 
            efind = find((cdfe2conde1(1:sizegrid-1)-onesqeps2(1:sizegrid-1)).*(cdfe2conde1(2:sizegrid)-onesqeps2(2:sizegrid))<0);
             
            if (isempty(efind))
                if (cdfe2conde1(1)-qeps2 < 0)
                    efind = 1;
                else
                    efind = sizegrid;
                end
            end
            
            eps2(ie) = grid(efind(1));
  
    end
    
    eps = [eps1 eps2];

    PP = unifrnd(-1,1,[necess,1]);

    save eps eps;

    else
        load eps;
    
        PP = unifrnd(-1,1,[necess,1]);
 
    end
    
	if ( dgp == DGP_LINEAR )
		YY = ( eps - [zeros([necess,1]) PP] ) * invAstar'; 
	elseif ( dgp == DGP_NONLIN )
        gammainv = Gamma^(-1);
        for ( i = 1:necess )
            YY(i,1) = fzero( @(y1) (delta2 * y1) - (gammainv * log((C/y1)-1)) + (eps(i,1) +eps(i,2)-PP(i)+delta1-d), [.00000000001  9.99999999999],options);    
        end
        YY(:,2) = delta1 * ones([necess,1]) + delta2 * YY(:,1) ...
                                                     - PP + eps(:,2);
        else
		error( 'Bad DGP specified; must be LINEAR or NONLIN' );
    end
    
	save( filename, 'PP', 'YY' );

